@@ -16,21 +16,33 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <stdlib.h>
+#include <stdint.h>
+#include <stdbool.h>
+#include <string.h>
+#include <unistd.h>
+
+#include "utils/config.h"
+#include "utils/errors.h"
+#include "utils/file.h"
+#include "utils/inet.h"
+#include "netsurf/bitmap.h"
 #include "content/hlcache.h"
 #include "content/backing_store.h"
 
-#include "utils/file.h"
 #include "desktop/save_pdf.h"
 #include "desktop/download.h"
 #include "desktop/searchweb.h"
-#include "desktop/gui_download.h"
-#include "desktop/gui_fetch.h"
-#include "desktop/gui_misc.h"
-#include "desktop/gui_window.h"
-#include "desktop/gui_search.h"
-#include "desktop/gui_clipboard.h"
-#include "desktop/gui_utf8.h"
-#include "desktop/netsurf.h"
+#include "netsurf/download.h"
+#include "netsurf/fetch.h"
+#include "netsurf/misc.h"
+#include "netsurf/window.h"
+#include "netsurf/core_window.h"
+#include "netsurf/search.h"
+#include "netsurf/clipboard.h"
+#include "netsurf/utf8.h"
+#include "netsurf/layout.h"
+#include "netsurf/netsurf.h"
 
 /**
  * The global interface table.
@@ -47,14 +59,6 @@ static nserror gui_default_window_set_url(struct gui_window *g, struct nsurl *ur
 	return NSERROR_OK;
 }
 
-static void gui_default_window_start_throbber(struct gui_window *g)
-{
-}
-
-static void gui_default_window_stop_throbber(struct gui_window *g)
-{
-}
-
 static bool gui_default_window_drag_start(struct gui_window *g,
 					  gui_drag_type type,
 					  const struct rect *rect)
@@ -74,23 +78,6 @@ static void gui_default_window_set_icon(struct gui_window *g,
 {
 }
 
-static void gui_default_window_scroll_visible(struct gui_window *g,
-				       int x0, int y0,
-				       int x1, int y1)
-{
-	guit->window->set_scroll(g, x0, y0);
-}
-
-static void gui_default_window_new_content(struct gui_window *g)
-{
-}
-
-
-static bool gui_default_window_scroll_start(struct gui_window *g)
-{
-	return true;
-}
-
 static void gui_default_window_set_pointer(struct gui_window *g,
 					   gui_pointer_shape shape)
 {
@@ -107,10 +94,6 @@ static void gui_default_window_place_caret(struct gui_window *g,
 {
 }
 
-static void gui_default_window_remove_caret(struct gui_window *g)
-{
-}
-
 static void gui_default_window_create_form_select_menu(struct gui_window *g,
 						struct form_control *control)
 {
@@ -133,7 +116,13 @@ static void gui_default_window_drag_save_selection(struct gui_window *g,
 {
 }
 
-static void gui_default_window_start_selection(struct gui_window *g)
+
+static void
+gui_default_console_log(struct gui_window *gw,
+			browser_window_console_source src,
+			const char *msg,
+			size_t msglen,
+			browser_window_console_flags flags)
 {
 }
 
@@ -153,10 +142,7 @@ static nserror verify_window_register(struct gui_window_table *gwt)
 	if (gwt->destroy == NULL) {
 		return NSERROR_BAD_PARAMETER;
 	}
-	if (gwt->redraw == NULL) {
-		return NSERROR_BAD_PARAMETER;
-	}
-	if (gwt->update == NULL) {
+	if (gwt->invalidate == NULL) {
 		return NSERROR_BAD_PARAMETER;
 	}
 	if (gwt->get_scroll == NULL) {
@@ -168,10 +154,7 @@ static nserror verify_window_register(struct gui_window_table *gwt)
 	if (gwt->get_dimensions == NULL) {
 		return NSERROR_BAD_PARAMETER;
 	}
-	if (gwt->update_extent == NULL) {
-		return NSERROR_BAD_PARAMETER;
-	}
-	if (gwt->reformat == NULL) {
+	if (gwt->event == NULL) {
 		return NSERROR_BAD_PARAMETER;
 	}
 
@@ -195,30 +178,12 @@ static nserror verify_window_register(struct gui_window_table *gwt)
 	if (gwt->place_caret == NULL) {
 		gwt->place_caret = gui_default_window_place_caret;
 	}
-	if (gwt->remove_caret == NULL) {
-		gwt->remove_caret = gui_default_window_remove_caret;
-	}
-	if (gwt->start_throbber == NULL) {
-		gwt->start_throbber = gui_default_window_start_throbber;
-	}
-	if (gwt->stop_throbber == NULL) {
-		gwt->stop_throbber = gui_default_window_stop_throbber;
-	}
 	if (gwt->drag_start == NULL) {
 		gwt->drag_start = gui_default_window_drag_start;
 	}
 	if (gwt->save_link == NULL) {
 		gwt->save_link = gui_default_window_save_link;
 	}
-	if (gwt->scroll_visible == NULL) {
-		gwt->scroll_visible = gui_default_window_scroll_visible;
-	}
-	if (gwt->new_content == NULL) {
-		gwt->new_content = gui_default_window_new_content;
-	}
-	if (gwt->scroll_start == NULL) {
-		gwt->scroll_start = gui_default_window_scroll_start;
-	}
 	if (gwt->create_form_select_menu == NULL) {
 		gwt->create_form_select_menu =
 				gui_default_window_create_form_select_menu;
@@ -232,14 +197,83 @@ static nserror verify_window_register(struct gui_window_table *gwt)
 	if (gwt->drag_save_selection == NULL) {
 		gwt->drag_save_selection = gui_default_window_drag_save_selection;
 	}
-	if (gwt->start_selection == NULL) {
-		gwt->start_selection = gui_default_window_start_selection;
+	if (gwt->console_log == NULL) {
+		gwt->console_log = gui_default_console_log;
 	}
 
 	return NSERROR_OK;
 }
 
 
+static nserror gui_default_corewindow_invalidate(struct core_window *cw, const struct rect *rect)
+{
+	return NSERROR_OK;
+}
+
+static nserror gui_default_corewindow_set_extent(struct core_window *cw, int width, int height)
+{
+	return NSERROR_OK;
+}
+
+static nserror gui_default_corewindow_set_scroll(struct core_window *cw, int x, int y)
+{
+	return NSERROR_OK;
+}
+
+static nserror gui_default_corewindow_get_scroll(const struct core_window *cw, int *x, int *y)
+{
+	return NSERROR_OK;
+}
+
+static nserror gui_default_corewindow_get_dimensions(const struct core_window *cw, int *width, int *height)
+{
+	return NSERROR_OK;
+}
+
+static nserror gui_default_corewindow_dragstatus(struct core_window *cw, core_window_drag_status ds)
+{
+	return NSERROR_OK;
+}
+
+static struct core_window_table default_corewindow_table = {
+	.invalidate = gui_default_corewindow_invalidate,
+	.set_extent = gui_default_corewindow_set_extent,
+	.set_scroll = gui_default_corewindow_set_scroll,
+	.get_scroll = gui_default_corewindow_get_scroll,
+	.get_dimensions = gui_default_corewindow_get_dimensions,
+	.drag_status = gui_default_corewindow_dragstatus,
+};
+
+/** verify corewindow window table is valid */
+static nserror verify_corewindow_register(struct core_window_table *cwt)
+{
+	/* check table is present */
+	if (cwt == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	/* all enties are mandantory */
+	if (cwt->invalidate == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+	if (cwt->set_extent == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+	if (cwt->set_scroll == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+	if (cwt->get_scroll == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+	if (cwt->get_dimensions == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+	if (cwt->drag_status == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	return NSERROR_OK;
+}
 
 static struct gui_download_window *
 gui_default_download_create(download_context *ctx, struct gui_window *parent)
@@ -495,11 +529,31 @@ static nsurl *gui_default_get_resource_url(const char *path)
 	return NULL;
 }
 
+static nserror gui_default_get_resource_data(const char *path, const uint8_t **data, size_t *data_len)
+{
+	return NSERROR_NOT_FOUND;
+}
+
+static nserror gui_default_release_resource_data(const uint8_t *data)
+{
+	return NSERROR_OK;
+}
+
 static char *gui_default_mimetype(const char *path)
 {
 	return strdup(guit->fetch->filetype(path));
 }
 
+static int gui_default_socket_open(int domain, int type, int protocol)
+{
+	return (int) socket(domain, type, protocol);
+}
+
+static int gui_default_socket_close(int fd)
+{
+	return (int) ns_close_socket(fd);
+}
+
 /** verify fetch table is valid */
 static nserror verify_fetch_register(struct gui_fetch_table *gft)
 {
@@ -517,9 +571,21 @@ static nserror verify_fetch_register(struct gui_fetch_table *gft)
 	if (gft->get_resource_url == NULL) {
 		gft->get_resource_url = gui_default_get_resource_url;
 	}
+	if (gft->get_resource_data == NULL) {
+		gft->get_resource_data = gui_default_get_resource_data;
+	}
+	if (gft->release_resource_data == NULL) {
+		gft->release_resource_data = gui_default_release_resource_data;
+	}
 	if (gft->mimetype == NULL) {
 		gft->mimetype = gui_default_mimetype;
 	}
+	if (gft->socket_open == NULL) {
+		gft->socket_open = gui_default_socket_open;
+	}
+	if (gft->socket_close == NULL) {
+		gft->socket_close = gui_default_socket_close;
+	}
 
 	return NSERROR_OK;
 }
@@ -552,6 +618,92 @@ static nserror verify_file_register(struct gui_file_table *gft)
 	return NSERROR_OK;
 }
 
+/**
+ * verify bitmap table is valid
+ *
+ * \param gbt The bitmap table to verify.
+ * \return NSERROR_OK if the table is valid else NSERROR_BAD_PARAMETER.
+ */
+static nserror verify_bitmap_register(struct gui_bitmap_table *gbt)
+{
+	/* check table is present */
+	if (gbt == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	/* check the mandantory fields are set */
+	if (gbt->create == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	if (gbt->destroy == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	if (gbt->set_opaque == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	if (gbt->get_opaque == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	if (gbt->get_buffer == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	if (gbt->get_rowstride == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	if (gbt->get_width == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	if (gbt->get_height == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	if (gbt->modified == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	if (gbt->render == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	return NSERROR_OK;
+}
+
+/**
+ * verify layout table is valid
+ *
+ * \param glt The layout table to verify.
+ * \return NSERROR_OK if the table is valid else NSERROR_BAD_PARAMETER.
+ */
+static nserror verify_layout_register(struct gui_layout_table *glt)
+{
+	/* check table is present */
+	if (glt == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	/* check the mandantory fields are set */
+	if (glt->width == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	if (glt->position == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	if (glt->split == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	return NSERROR_OK;
+}
+
 static void gui_default_quit(void)
 {
 }
@@ -563,19 +715,16 @@ static nserror gui_default_launch_url(struct nsurl *url)
 }
 
 
-static void gui_default_cert_verify(nsurl *url,
-				    const struct ssl_cert_info *certs,
-				    unsigned long num,
-				    nserror (*cb)(bool proceed, void *pw),
-				    void *cbpw)
+static nserror gui_default_401login_open(
+	nsurl *url, const char *realm,
+	const char *username, const char *password,
+	nserror (*cb)(nsurl *url, const char * realm,
+		      const char *username,
+		      const char *password,
+		      void *pw),
+	void *cbpw)
 {
-	cb(false, cbpw);
-}
-
-static void gui_default_401login_open(nsurl *url, const char *realm,
-		nserror (*cb)(bool proceed, void *pw), void *cbpw)
-{
-	cb(false, cbpw);
+	return NSERROR_NOT_IMPLEMENTED;
 }
 
 static void
@@ -585,40 +734,46 @@ gui_default_pdf_password(char **owner_pass, char **user_pass, char *path)
 	save_pdf(path);
 }
 
-/** verify browser table is valid */
-static nserror verify_browser_register(struct gui_browser_table *gbt)
+static nserror
+gui_default_present_cookies(const char *search_term)
+{
+	return NSERROR_NOT_IMPLEMENTED;
+}
+
+/** verify misc table is valid */
+static nserror verify_misc_register(struct gui_misc_table *gmt)
 {
 	/* check table is present */
-	if (gbt == NULL) {
+	if (gmt == NULL) {
 		return NSERROR_BAD_PARAMETER;
 	}
 
 	/* check the mandantory fields are set */
-	if (gbt->schedule == NULL) {
+	if (gmt->schedule == NULL) {
 		return NSERROR_BAD_PARAMETER;
 	}
 
 	/* fill in the optional entries with defaults */
-	if (gbt->quit == NULL) {
-		gbt->quit = gui_default_quit;
+	if (gmt->quit == NULL) {
+		gmt->quit = gui_default_quit;
 	}
-	if (gbt->launch_url == NULL) {
-		gbt->launch_url = gui_default_launch_url;
+	if (gmt->launch_url == NULL) {
+		gmt->launch_url = gui_default_launch_url;
 	}
-	if (gbt->cert_verify == NULL) {
-		gbt->cert_verify = gui_default_cert_verify;
+	if (gmt->login == NULL) {
+		gmt->login = gui_default_401login_open;
 	}
-	if (gbt->login == NULL) {
-		gbt->login = gui_default_401login_open;
+	if (gmt->pdf_password == NULL) {
+		gmt->pdf_password = gui_default_pdf_password;
 	}
-	if (gbt->pdf_password == NULL) {
-		gbt->pdf_password = gui_default_pdf_password;
+	if (gmt->present_cookies == NULL) {
+		gmt->present_cookies = gui_default_present_cookies;
 	}
 	return NSERROR_OK;
 }
 
 
-/* exported interface documented in desktop/netsurf.h */
+/* exported interface documented in netsurf/netsurf.h */
 nserror netsurf_register(struct netsurf_table *gt)
 {
 	nserror err;
@@ -633,8 +788,10 @@ nserror netsurf_register(struct netsurf_table *gt)
 		return NSERROR_BAD_PARAMETER;
 	}
 
-	/* browser table */
-	err = verify_browser_register(gt->browser);
+	/* mandantory tables */
+
+	/* miscellaneous table */
+	err = verify_misc_register(gt->misc);
 	if (err != NSERROR_OK) {
 		return err;
 	}
@@ -651,6 +808,31 @@ nserror netsurf_register(struct netsurf_table *gt)
 		return err;
 	}
 
+	/* bitmap table */
+	err = verify_bitmap_register(gt->bitmap);
+	if (err != NSERROR_OK) {
+		return err;
+	}
+
+	/* layout table */
+	err = verify_layout_register(gt->layout);
+	if (err != NSERROR_OK) {
+		return err;
+	}
+
+	/* optional tables */
+
+	/* core window table */
+	if (gt->corewindow == NULL) {
+		/* set default core window table */
+		gt->corewindow = &default_corewindow_table;
+	}
+	err = verify_corewindow_register(gt->corewindow);
+	if (err != NSERROR_OK) {
+		return err;
+	}
+
+
 	/* file table */
 	if (gt->file == NULL) {
 		gt->file = default_file_table;
