@@ -27,7 +27,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
-#include <libutf8proc/utf8proc.h>
 
 #include "netsurf/inttypes.h"
 
@@ -40,14 +39,6 @@
 #include "utils/utils.h"
 
 
-int32_t idna_contexto[] = {
-	/* CONTEXTO codepoints which have a rule defined */
-	0x00b7, 0x0375, 0x05f3, 0x05f4, 0x30fb, 0x0660, 0x0661,
-	0x0662, 0x0663, 0x0664, 0x0665, 0x0666, 0x0667, 0x0668,
-	0x0669, 0x06f0, 0x06f1, 0x06f2, 0x06f3, 0x06f4, 0x06f5,
-	0x06f6, 0x06f7, 0x06f8, 0x06f9, 0
-};
-
 /**
  * Convert punycode status into nserror.
  *
@@ -84,6 +75,108 @@ static nserror punycode_status_to_nserror(enum punycode_status status)
 	return ret;
 }
 
+
+/**
+ * Convert a host label in UCS-4 to an ACE version
+ *
+ * \param ucs4_label UCS-4 NFC string containing host label
+ * \param len Length of host label (in characters/codepoints)
+ * \param ace_label ASCII-compatible encoded version
+ * \param out_len Length of ace_label
+ * \return NSERROR_OK on success, appropriate error otherwise
+ *
+ * If return value != NSERROR_OK, output will be left untouched.
+ */
+static nserror
+idna__ucs4_to_ace(int32_t *ucs4_label,
+		  size_t len,
+		  char **ace_label,
+		  size_t *out_len)
+{
+	char punycode[65]; /* max length of host label + NULL */
+	size_t output_length = 60; /* punycode length - 4 - 1 */
+	nserror ret;
+
+	punycode[0] = 'x';
+	punycode[1] = 'n';
+	punycode[2] = '-';
+	punycode[3] = '-';
+
+	ret = punycode_status_to_nserror(punycode_encode(len,
+			(const punycode_uint *)ucs4_label, NULL,
+			&output_length, punycode + 4));
+	if (ret != NSERROR_OK) {
+		return ret;
+	}
+
+	output_length += SLEN("xn--");
+	punycode[output_length] = '\0';
+
+	*ace_label = strdup(punycode);
+	*out_len = output_length;
+
+	return NSERROR_OK;
+}
+
+
+/**
+ * Convert a host label in ACE format to UCS-4
+ *
+ * \param ace_label ASCII string containing host label
+ * \param ace_len Length of host label
+ * \param ucs4_label Pointer to hold UCS4 decoded version
+ * \param ucs4_len Pointer to hold length of ucs4_label
+ * \return NSERROR_OK on success, appropriate error otherwise
+ *
+ * If return value != NSERROR_OK, output will be left untouched.
+ */
+static nserror
+idna__ace_to_ucs4(const char *ace_label,
+		  size_t ace_len,
+		  int32_t **ucs4_label,
+		  size_t *ucs4_len)
+{
+	int32_t *ucs4;
+	nserror ret;
+	size_t output_length = ace_len; /* never exceeds input length */
+
+	/* The header should always have been checked before calling */
+	assert((ace_label[0] == 'x') && (ace_label[1] == 'n') &&
+		(ace_label[2] == '-') && (ace_label[3] == '-'));
+
+	ucs4 = malloc(output_length * 4);
+	if (ucs4 == NULL) {
+		return NSERROR_NOMEM;
+	}
+
+	ret = punycode_status_to_nserror(punycode_decode(ace_len - 4,
+		ace_label + 4, &output_length, (punycode_uint *)ucs4, NULL));
+	if (ret != NSERROR_OK) {
+		free(ucs4);
+		return ret;
+	}
+
+	ucs4[output_length] = '\0';
+
+	*ucs4_label = ucs4;
+	*ucs4_len = output_length;
+
+	return NSERROR_OK;
+}
+
+
+#ifdef WITH_UTF8PROC
+
+#include <libutf8proc/utf8proc.h>
+
+int32_t idna_contexto[] = {
+	/* CONTEXTO codepoints which have a rule defined */
+	0x00b7, 0x0375, 0x05f3, 0x05f4, 0x30fb, 0x0660, 0x0661,
+	0x0662, 0x0663, 0x0664, 0x0665, 0x0666, 0x0667, 0x0668,
+	0x0669, 0x06f0, 0x06f1, 0x06f2, 0x06f3, 0x06f4, 0x06f5,
+	0x06f6, 0x06f7, 0x06f8, 0x06f9, 0
+};
+
 /**
  * Find the IDNA property of a UCS-4 codepoint
  *
@@ -306,94 +399,217 @@ idna__ucs4_to_utf8(const int32_t *ucs4_label,
 
 
 /**
- * Convert a host label in UCS-4 to an ACE version
+ * Check if a host label is valid for IDNA2008
  *
- * \param ucs4_label	UCS-4 NFC string containing host label
+ * \param label	Host label to check (UCS-4)
  * \param len	Length of host label (in characters/codepoints)
- * \param ace_label	ASCII-compatible encoded version
- * \param out_len	Length of ace_label
- * \return NSERROR_OK on success, appropriate error otherwise
- *
- * If return value != NSERROR_OK, output will be left untouched.
+ * \return true if compliant, false otherwise
  */
-static nserror
-idna__ucs4_to_ace(int32_t *ucs4_label,
-		  size_t len,
-		  char **ace_label,
-		  size_t *out_len)
+static bool idna__is_valid(int32_t *label, size_t len)
 {
-	char punycode[65]; /* max length of host label + NULL */
-	size_t output_length = 60; /* punycode length - 4 - 1 */
-	nserror ret;
+	const utf8proc_property_t *unicode_props;
+	idna_property idna_prop;
+	size_t i = 0;
 
-	punycode[0] = 'x';
-	punycode[1] = 'n';
-	punycode[2] = '-';
-	punycode[3] = '-';
+	/* 1. Check that the string is NFC.
+	 * This check is skipped as the conversion to Unicode
+	 * does normalisation as part of the conversion.
+	 */
 
-	ret = punycode_status_to_nserror(punycode_encode(len,
-			(const punycode_uint *)ucs4_label, NULL,
-			&output_length, punycode + 4));
-	if (ret != NSERROR_OK) {
-		return ret;
+	/* 2. Check characters 3 and 4 are not '--'. */
+	if ((len >= 4) && (label[2] == 0x002d) && (label[3] == 0x002d)) {
+		NSLOG(netsurf, INFO,
+		      "Check failed: characters 2 and 3 are '--'");
+		return false;
 	}
 
-	output_length += SLEN("xn--");
-	punycode[output_length] = '\0';
+	/* 3. Check the first character is not a combining mark */
+	unicode_props = utf8proc_get_property(label[0]);
 
-	*ace_label = strdup(punycode);
-	*out_len = output_length;
+	if ((unicode_props->category == UTF8PROC_CATEGORY_MN) ||
+		(unicode_props->category == UTF8PROC_CATEGORY_MC) ||
+		(unicode_props->category == UTF8PROC_CATEGORY_ME)) {
+		NSLOG(netsurf, INFO,
+		      "Check failed: character 0 is a combining mark");
+		return false;
+	}
 
-	return NSERROR_OK;
+	for (i = 0; i < len; i++) {
+		idna_prop = idna__cp_property(label[i]);
+
+		/* 4. Check characters not DISALLOWED by RFC5892 */
+		if (idna_prop == IDNA_P_DISALLOWED) {
+			NSLOG(netsurf, INFO,
+			      "Check failed: character %"PRIsizet" (%"PRIx32") is DISALLOWED",
+			      i,
+			      label[i]);
+			return false;
+		}
+
+		/* 5. Check CONTEXTJ characters conform to defined rules */
+		if (idna_prop == IDNA_P_CONTEXTJ) {
+			if (idna__contextj_rule(label, i, len) == false) {
+				NSLOG(netsurf, INFO,
+				      "Check failed: character %"PRIsizet" (%"PRIx32") does not conform to CONTEXTJ rule",
+				      i,
+				      label[i]);
+				return false;
+			}
+		}
+
+		/* 6. Check CONTEXTO characters have a rule defined */
+		/** \todo optionally we can check conformance to this rule */
+		if (idna_prop == IDNA_P_CONTEXTO) {
+			if (idna__contexto_rule(label[i]) == false) {
+				NSLOG(netsurf, INFO,
+				      "Check failed: character %"PRIsizet" (%"PRIx32") has no CONTEXTO rule defined",
+				      i,
+				      label[i]);
+				return false;
+			}
+		}
+
+		/* 7. Check characters are not UNASSIGNED */
+		if (idna_prop == IDNA_P_UNASSIGNED) {
+			NSLOG(netsurf, INFO,
+			      "Check failed: character %"PRIsizet" (%"PRIx32") is UNASSIGNED",
+			      i,
+			      label[i]);
+			return false;
+		}
+
+		/** \todo 8. (optionally) check Bidi compliance */
+	}
+
+	return true;
 }
 
 
 /**
- * Convert a host label in ACE format to UCS-4
+ * Verify an ACE label is valid
  *
- * \param ace_label	ASCII string containing host label
- * \param ace_len	Length of host label
- * \param ucs4_label	Pointer to hold UCS4 decoded version
- * \param ucs4_len	Pointer to hold length of ucs4_label
+ * \param label	Host label to check
+ * \param len	Length of label
+ * \return true if valid, false otherwise
+ */
+static bool idna__verify(const char *label, size_t len)
+{
+	nserror error;
+	int32_t *ucs4;
+	char *ace;
+	ssize_t ucs4_len;
+	size_t u_ucs4_len, ace_len;
+
+	/* Convert our ACE label back to UCS-4 */
+	error = idna__ace_to_ucs4(label, len, &ucs4, &u_ucs4_len);
+	if (error != NSERROR_OK) {
+		return false;
+	}
+
+	/* Perform NFC normalisation */
+	ucs4_len = utf8proc_normalize_utf32(ucs4, u_ucs4_len,
+		UTF8PROC_STABLE | UTF8PROC_COMPOSE);
+	if (ucs4_len < 0) {
+		free(ucs4);
+		return false;
+	}
+
+	/* Convert the UCS-4 label back to ACE */
+	error = idna__ucs4_to_ace(ucs4, (size_t)ucs4_len,
+				&ace, &ace_len);
+	free(ucs4);
+	if (error != NSERROR_OK) {
+		return false;
+	}
+
+	/* Check if it matches the input */
+	if ((len == ace_len) && (strncmp(label, ace, len) == 0)) {
+		free(ace);
+		return true;
+	}
+
+	NSLOG(netsurf, INFO, "Re-encoded ACE label %s does not match input",
+	      ace);
+	free(ace);
+
+	return false;
+}
+
+
+#else /* WITH_UTF8PROC */
+
+
+/**
+ * Convert a UTF-8 string to UCS-4
+ *
+ * \param utf8_label	UTF-8 string containing host label
+ * \param len	Length of host label (in bytes)
+ * \param ucs4_label	Pointer to update with the output
+ * \param ucs4_len	Pointer to update with the length
  * \return NSERROR_OK on success, appropriate error otherwise
  *
  * If return value != NSERROR_OK, output will be left untouched.
  */
 static nserror
-idna__ace_to_ucs4(const char *ace_label,
-		  size_t ace_len,
-		  int32_t **ucs4_label,
-		  size_t *ucs4_len)
+idna__utf8_to_ucs4(const char *utf8_label,
+		   size_t len,
+		   int32_t **ucs4_label,
+		   size_t *ucs4_len)
 {
-	int32_t *ucs4;
-	nserror ret;
-	size_t output_length = ace_len; /* never exceeds input length */
-
-	/* The header should always have been checked before calling */
-	assert((ace_label[0] == 'x') && (ace_label[1] == 'n') &&
-		(ace_label[2] == '-') && (ace_label[3] == '-'));
-
-	ucs4 = malloc(output_length * 4);
-	if (ucs4 == NULL) {
-		return NSERROR_NOMEM;
-	}
-
-	ret = punycode_status_to_nserror(punycode_decode(ace_len - 4,
-		ace_label + 4, &output_length, (punycode_uint *)ucs4, NULL));
-	if (ret != NSERROR_OK) {
-		free(ucs4);
-		return ret;
-	}
-
-	ucs4[output_length] = '\0';
-
-	*ucs4_label = ucs4;
-	*ucs4_len = output_length;
-
-	return NSERROR_OK;
+	return NSERROR_NOT_IMPLEMENTED;
 }
 
 
+/**
+ * Convert a UCS-4 string to UTF-8
+ *
+ * \param ucs4_label	UCS-4 string containing host label
+ * \param ucs4_len	Length of host label (in bytes)
+ * \param utf8_label	Pointer to update with the output
+ * \param utf8_len	Pointer to update with the length
+ * \return NSERROR_OK on success, appropriate error otherwise
+ *
+ * If return value != NSERROR_OK, output will be left untouched.
+ */
+static nserror
+idna__ucs4_to_utf8(const int32_t *ucs4_label,
+		   size_t ucs4_len,
+		   char **utf8_label,
+		   size_t *utf8_len)
+{
+	return NSERROR_NOT_IMPLEMENTED;
+}
+
+
+/**
+ * Check if a host label is valid for IDNA2008
+ *
+ * \param label	Host label to check (UCS-4)
+ * \param len	Length of host label (in characters/codepoints)
+ * \return true if compliant, false otherwise
+ */
+static bool idna__is_valid(int32_t *label, size_t len)
+{
+	return true;
+}
+
+
+/**
+ * Verify an ACE label is valid
+ *
+ * \param label	Host label to check
+ * \param len	Length of label
+ * \return true if valid, false otherwise
+ */
+static bool idna__verify(const char *label, size_t len)
+{
+	return true;
+}
+
+
+#endif /* WITH_UTF8PROC */
+
+
 /**
  * Find the length of a host label
  *
@@ -418,93 +634,6 @@ static size_t idna__host_label_length(const char *host, size_t max_length)
 }
 
 
-/**
- * Check if a host label is valid for IDNA2008
- *
- * \param label	Host label to check (UCS-4)
- * \param len	Length of host label (in characters/codepoints)
- * \return true if compliant, false otherwise
- */
-static bool idna__is_valid(int32_t *label, size_t len)
-{
-	const utf8proc_property_t *unicode_props;
-	idna_property idna_prop;
-	size_t i = 0;
-
-	/* 1. Check that the string is NFC.
-	 * This check is skipped as the conversion to Unicode
-	 * does normalisation as part of the conversion.
-	 */
-
-	/* 2. Check characters 3 and 4 are not '--'. */
-	if ((label[2] == 0x002d) && (label[3] == 0x002d)) {
-		NSLOG(netsurf, INFO,
-		      "Check failed: characters 2 and 3 are '--'");
-		return false;
-	}
-
-	/* 3. Check the first character is not a combining mark */
-	unicode_props = utf8proc_get_property(label[0]);
-
-	if ((unicode_props->category == UTF8PROC_CATEGORY_MN) ||
-		(unicode_props->category == UTF8PROC_CATEGORY_MC) ||
-		(unicode_props->category == UTF8PROC_CATEGORY_ME)) {
-		NSLOG(netsurf, INFO,
-		      "Check failed: character 0 is a combining mark");
-		return false;
-	}
-
-	for (i = 0; i < len; i++) {
-		idna_prop = idna__cp_property(label[i]);
-
-		/* 4. Check characters not DISALLOWED by RFC5892 */
-		if (idna_prop == IDNA_P_DISALLOWED) {
-			NSLOG(netsurf, INFO,
-			      "Check failed: character %"PRIsizet" (%x) is DISALLOWED",
-			      i,
-			      label[i]);
-			return false;
-		}
-
-		/* 5. Check CONTEXTJ characters conform to defined rules */
-		if (idna_prop == IDNA_P_CONTEXTJ) {
-			if (idna__contextj_rule(label, i, len) == false) {
-				NSLOG(netsurf, INFO,
-				      "Check failed: character %"PRIsizet" (%x) does not conform to CONTEXTJ rule",
-				      i,
-				      label[i]);
-				return false;
-			}
-		}
-
-		/* 6. Check CONTEXTO characters have a rule defined */
-		/** \todo optionally we can check conformance to this rule */
-		if (idna_prop == IDNA_P_CONTEXTO) {
-			if (idna__contexto_rule(label[i]) == false) {
-				NSLOG(netsurf, INFO,
-				      "Check failed: character %"PRIsizet" (%x) has no CONTEXTO rule defined",
-				      i,
-				      label[i]);
-				return false;
-			}
-		}
-
-		/* 7. Check characters are not UNASSIGNED */
-		if (idna_prop == IDNA_P_UNASSIGNED) {
-			NSLOG(netsurf, INFO,
-			      "Check failed: character %"PRIsizet" (%x) is UNASSIGNED",
-			      i,
-			      label[i]);
-			return false;
-		}
-
-		/** \todo 8. (optionally) check Bidi compliance */
-	}
-
-	return true;
-}
-
-
 /**
  * Check if a host label is LDH
  *
@@ -559,57 +688,25 @@ static bool idna__is_ace(const char *label, size_t len)
 	return false;
 }
 
+/* This is the maximum length of a full DNS name */
+#define FQDN_MAX 256
 
-/**
- * Verify an ACE label is valid
- *
- * \param label	Host label to check
- * \param len	Length of label
- * \return true if valid, false otherwise
- */
-static bool idna__verify(const char *label, size_t len)
-{
-	nserror error;
-	int32_t *ucs4;
-	char *ace;
-	ssize_t ucs4_len;
-	size_t u_ucs4_len, ace_len;
-
-	/* Convert our ACE label back to UCS-4 */
-	error = idna__ace_to_ucs4(label, len, &ucs4, &u_ucs4_len);
-	if (error != NSERROR_OK) {
-		return false;
-	}
-
-	/* Perform NFC normalisation */
-	ucs4_len = utf8proc_normalize_utf32(ucs4, u_ucs4_len,
-		UTF8PROC_STABLE | UTF8PROC_COMPOSE);
-	if (ucs4_len < 0) {
-		free(ucs4);
-		return false;
-	}
-
-	/* Convert the UCS-4 label back to ACE */
-	error = idna__ucs4_to_ace(ucs4, (size_t)ucs4_len,
-				&ace, &ace_len);
-	free(ucs4);
-	if (error != NSERROR_OK) {
-		return false;
-	}
-
-	/* Check if it matches the input */
-	if ((len == ace_len) && (strncmp(label, ace, len) == 0)) {
-		free(ace);
-		return true;
-	}
-
-	NSLOG(netsurf, INFO, "Re-encoded ACE label %s does not match input",
-	      ace);
-	free(ace);
-
-	return false;
-}
+/* A "no action" action for FQDN_APPEND */
+#define NO_ACTION (void)0
 
+#define FQDN_APPEND(s, len, action)                                            \
+	do {                                                                   \
+		if ((FQDN_MAX - fqdn_len) <= len) {                            \
+			/* Not enough room to append this element */           \
+			action;                                                \
+			return NSERROR_BAD_URL;                                \
+		} else {                                                       \
+			memcpy(fqdn_p, s, len);                                \
+			fqdn_p += len;                                         \
+			fqdn_len += len;                                       \
+			action;                                                \
+		}                                                              \
+	} while (0)
 
 /* exported interface documented in idna.h */
 nserror
@@ -618,7 +715,7 @@ idna_encode(const char *host, size_t len, char **ace_host, size_t *ace_len)
 	nserror error;
 	int32_t *ucs4_host;
 	size_t label_len, output_len, ucs4_len, fqdn_len = 0;
-	char fqdn[256];
+	char fqdn[FQDN_MAX];
 	char *output, *fqdn_p = fqdn;
 
 	label_len = idna__host_label_length(host, len);
@@ -631,8 +728,9 @@ idna_encode(const char *host, size_t len, char **ace_host, size_t *ace_len)
 			/* This string is IDN or invalid */
 
 			/* Convert to Unicode */
-			if ((error = idna__utf8_to_ucs4(host, label_len,
-					&ucs4_host, &ucs4_len)) != NSERROR_OK) {
+			error = idna__utf8_to_ucs4(host, label_len,
+						   &ucs4_host, &ucs4_len);
+			if (error != NSERROR_OK) {
 				return error;
 			}
 
@@ -649,10 +747,7 @@ idna_encode(const char *host, size_t len, char **ace_host, size_t *ace_len)
 			if (error != NSERROR_OK) {
 				return error;
 			}
-			strncpy(fqdn_p, output, output_len);
-			free(output);
-			fqdn_p += output_len;
-			fqdn_len += output_len;
+			FQDN_APPEND(output, output_len, free(output));
 		} else {
 			/* This is already a DNS-valid ASCII string */
 			if ((idna__is_ace(host, label_len) == true) &&
@@ -661,14 +756,10 @@ idna_encode(const char *host, size_t len, char **ace_host, size_t *ace_len)
 				      "Cannot verify ACE label %s", host);
 				return NSERROR_BAD_URL;
 			}
-			strncpy(fqdn_p, host, label_len);
-			fqdn_p += label_len;
-			fqdn_len += label_len;
+			FQDN_APPEND(host, label_len, NO_ACTION);
 		}
 
-		*fqdn_p = '.';
-		fqdn_p++;
-		fqdn_len++;
+		FQDN_APPEND(".", 1, NO_ACTION);
 
 		host += label_len;
 		if ((*host == '\0') || (*host == ':')) {
@@ -696,7 +787,7 @@ idna_decode(const char *ace_host, size_t ace_len, char **host, size_t *host_len)
 	nserror error;
 	int32_t *ucs4_host;
 	size_t label_len, output_len, ucs4_len, fqdn_len = 0;
-	char fqdn[256];
+	char fqdn[FQDN_MAX];
 	char *output, *fqdn_p = fqdn;
 
 	label_len = idna__host_label_length(ace_host, ace_len);
@@ -710,7 +801,7 @@ idna_decode(const char *ace_host, size_t ace_len, char **host, size_t *host_len)
 
 			/* Decode to Unicode */
 			error = idna__ace_to_ucs4(ace_host, label_len,
-					&ucs4_host, &ucs4_len);
+						  &ucs4_host, &ucs4_len);
 			if (error != NSERROR_OK) {
 				return error;
 			}
@@ -722,21 +813,13 @@ idna_decode(const char *ace_host, size_t ace_len, char **host, size_t *host_len)
 			if (error != NSERROR_OK) {
 				return error;
 			}
-
-			memcpy(fqdn_p, output, output_len * 4);
-			free(output);
-			fqdn_p += output_len;
-			fqdn_len += output_len;
+			FQDN_APPEND(output, output_len, free(output));
 		} else {
 			/* Not ACE */
-			memcpy(fqdn_p, ace_host, label_len);
-			fqdn_p += label_len;
-			fqdn_len += label_len;
+			FQDN_APPEND(ace_host, label_len, NO_ACTION);
 		}
 
-		*fqdn_p = '.';
-		fqdn_p++;
-		fqdn_len++;
+		FQDN_APPEND(".", 1, NO_ACTION);
 
 		ace_host += label_len;
 		if ((*ace_host == '\0') || (*ace_host == ':')) {
