@@ -16,7 +16,8 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/** \file
+/**
+ * \file
  * Option reading and saving (implementation).
  *
  * Options are stored in the format key:value, one per line.
@@ -31,12 +32,16 @@
 #include <string.h>
 #include <strings.h>
 
-#include "desktop/plot_style.h"
+#include "netsurf/inttypes.h"
+#include "netsurf/plot_style.h"
 #include "utils/errors.h"
 #include "utils/log.h"
 #include "utils/utils.h"
 #include "utils/nsoption.h"
 
+/** Length of buffer used to read lines from input file */
+#define NSOPTION_MAX_LINE_LEN 1024
+
 struct nsoption_s *nsoptions = NULL;
 struct nsoption_s *nsoptions_default = NULL;
 
@@ -73,6 +78,8 @@ static struct nsoption_s defaults[] = {
 #include "atari/options.h"
 #elif defined(nsmonkey)
 #include "monkey/options.h"
+#elif defined(nswin32)
+#include "windows/options.h"
 #endif
 	{ NULL, 0, OPTION_INTEGER, { 0 } }
 };
@@ -106,7 +113,7 @@ strtooption(const char *value, struct nsoption_s *option)
 		break;
 
 	case OPTION_COLOUR:
-		if (sscanf(value, "%x", &rgbcolour) == 1) {
+		if (sscanf(value, "%"SCNx32"", &rgbcolour) == 1) {
 			option->value.c = (((0x000000FF & rgbcolour) << 16) |
 					   ((0x0000FF00 & rgbcolour) << 0) |
 					   ((0x00FF0000 & rgbcolour) >> 16));
@@ -140,6 +147,14 @@ static void nsoption_validate(struct nsoption_s *opts, struct nsoption_s *defs)
 	int cloop;
 	bool black = true;
 
+	if (opts[NSOPTION_treeview_font_size].value.i  < 50) {
+		opts[NSOPTION_treeview_font_size].value.i = 50;
+	}
+
+	if (opts[NSOPTION_treeview_font_size].value.i > 1000) {
+		opts[NSOPTION_treeview_font_size].value.i = 1000;
+	}
+
 	if (opts[NSOPTION_font_size].value.i  < 50) {
 		opts[NSOPTION_font_size].value.i = 50;
 	}
@@ -173,13 +188,31 @@ static void nsoption_validate(struct nsoption_s *opts, struct nsoption_s *defs)
 			break;
 		}
 	}
-	if (black == true) {
+	if (black == true && defs != NULL) {
 		for (cloop = NSOPTION_SYS_COLOUR_START;
 		     cloop <= NSOPTION_SYS_COLOUR_END;
 		     cloop++) {
 			opts[cloop].value.c = defs[cloop].value.c;
 		}
 	}
+
+	/* To aid migration and ensure that timeouts don't go crazy,
+	 * ensure that (a) we allow at least 1 attempt and
+	 * (b) the total time that we spend should not exceed 60s
+	 */
+	if (opts[NSOPTION_max_retried_fetches].value.u == 0)
+		opts[NSOPTION_max_retried_fetches].value.u = 1;
+	if (opts[NSOPTION_curl_fetch_timeout].value.u < 5)
+		opts[NSOPTION_curl_fetch_timeout].value.u = 5;
+	if (opts[NSOPTION_curl_fetch_timeout].value.u > 60)
+		opts[NSOPTION_curl_fetch_timeout].value.u = 60;
+	while (((opts[NSOPTION_curl_fetch_timeout].value.u *
+		 opts[NSOPTION_max_retried_fetches].value.u) > 60) &&
+		(opts[NSOPTION_max_retried_fetches].value.u > 1))
+		opts[NSOPTION_max_retried_fetches].value.u--;
+
+	/* We ignore the result because we can't fail to validate. Yay */
+	(void)nslog_set_filter_by_options();
 }
 
 /**
@@ -232,6 +265,8 @@ nsoption_is_set(const struct nsoption_s *opts,
 		 */
 		if (((defs[entry].value.s == NULL) &&
 		     (opts[entry].value.s != NULL)) ||
+		    ((defs[entry].value.s != NULL) &&
+		     (opts[entry].value.s == NULL)) ||
 		    ((defs[entry].value.s != opts[entry].value.s) &&
 		     (strcmp(opts[entry].value.s, defs[entry].value.s) != 0))) {
 			ret = true;
@@ -289,7 +324,7 @@ nsoption_output(FILE *fp,
 			rgbcolour = (((0x000000FF & opts[entry].value.c) << 16) |
 				     ((0x0000FF00 & opts[entry].value.c) << 0) |
 				     ((0x00FF0000 & opts[entry].value.c) >> 16));
-			fprintf(fp, "%s:%06x\n",
+			fprintf(fp, "%s:%06"PRIx32"\n",
 				opts[entry].key,
 				rgbcolour);
 
@@ -349,17 +384,20 @@ nsoption_output_value_html(struct nsoption_s *option,
 		break;
 
 	case OPTION_COLOUR:
-		rgbcolour = (((0x000000FF & option->value.c) << 16) |
-			     ((0x0000FF00 & option->value.c) << 0) |
-			     ((0x00FF0000 & option->value.c) >> 16));
+		rgbcolour = colour_rb_swap(option->value.c);
 		slen = snprintf(string + pos,
 				size - pos,
-				"<span style=\"background-color: #%06x; "
-				"color: #%06x; "
-				"font-family:Monospace; \">#%06X</span>",
+				"<span style=\"font-family:Monospace;\">"
+					"#%06"PRIX32
+				"</span> "
+				"<span style=\"background-color: #%06"PRIx32"; "
+					"border: 1px solid #%06"PRIx32"; "
+					"display: inline-block; "
+					"width: 1em; height: 1em;\">"
+				"</span>",
 				rgbcolour,
-				colour_to_bw_furthest(rgbcolour),
-				rgbcolour);
+				rgbcolour,
+				colour_to_bw_furthest(rgbcolour));
 		break;
 
 	case OPTION_STRING:
@@ -422,7 +460,7 @@ nsoption_output_value_text(struct nsoption_s *option,
 		rgbcolour = (((0x000000FF & option->value.c) << 16) |
 			     ((0x0000FF00 & option->value.c) << 0) |
 			     ((0x00FF0000 & option->value.c) >> 16));
-		slen = snprintf(string + pos, size - pos, "%06x", rgbcolour);
+		slen = snprintf(string + pos, size - pos, "%06"PRIx32, rgbcolour);
 		break;
 
 	case OPTION_STRING:
@@ -503,6 +541,89 @@ nsoption_free(struct nsoption_s *opts)
 }
 
 
+/**
+ * extract key/value from a line of input
+ *
+ * \retun NSERROR_OK and key_out and value_out updated
+ *        NSERROR_NOT_FOUND if not a key/value input line
+ *        NSERROR_INVALID if the line is and invalid format (missing colon)
+ */
+static nserror
+get_key_value(char *line, int linelen, char **key_out, char **value_out)
+{
+	char *key;
+	char *value;
+
+	/* skip leading whitespace for start of key */
+	for (key = line; *key != 0; key++) {
+		if ((*key != ' ') && (*key != '\t') && (*key != '\n')) {
+			break;
+		}
+	}
+
+	/* empty line or only whitespace */
+	if (*key == 0) {
+		return NSERROR_NOT_FOUND;
+	}
+
+	/* comment */
+	if (*key == '#') {
+		return NSERROR_NOT_FOUND;
+	}
+
+	/* get start of value */
+	for (value = key; *value != 0; value++) {
+		if (*value == ':') {
+			*value = 0;
+			value++;
+			break;
+		}
+	}
+
+	/* missing colon separator */
+	if (*value == 0) {
+		return NSERROR_INVALID;
+	}
+
+	/* remove delimiter from value */
+	if (line[linelen - 1] == '\n') {
+		linelen--;
+		line[linelen] = 0;
+	}
+
+	*key_out = key;
+	*value_out = value;
+	return NSERROR_OK;
+}
+
+
+/**
+ * Process a line from a user option file
+ */
+static nserror optionline(struct nsoption_s *opts, char *line, int linelen)
+{
+	nserror res;
+	char *key;
+	char *value;
+	int idx;
+
+	res = get_key_value(line, linelen, &key, &value);
+	if (res != NSERROR_OK) {
+		/* skip line as no valid key value pair found */
+		return res;
+	}
+
+	for (idx = 0; opts[idx].key != NULL; idx++) {
+		if (strcasecmp(key, opts[idx].key) == 0) {
+			strtooption(value, &opts[idx]);
+			break;
+		}
+	}
+
+	return res;
+}
+
+
 /* exported interface documented in utils/nsoption.h */
 nserror
 nsoption_init(nsoption_set_default_t *set_defaults,
@@ -562,28 +683,40 @@ nsoption_init(nsoption_set_default_t *set_defaults,
 /* exported interface documented in utils/nsoption.h */
 nserror nsoption_finalise(struct nsoption_s *opts, struct nsoption_s *defs)
 {
+	nserror res;
+
 	/* check to see if global table selected */
 	if (opts == NULL) {
-		opts = nsoptions;
+		res = nsoption_free(nsoptions);
+		if (res == NSERROR_OK) {
+			nsoptions = NULL;
+		}
+	} else {
+		res = nsoption_free(opts);
+	}
+	if (res != NSERROR_OK) {
+		return res;
 	}
-
-	nsoption_free(opts);
 
 	/* check to see if global table selected */
 	if (defs == NULL) {
-		defs = nsoptions_default;
+		res = nsoption_free(nsoptions_default);
+		if (res == NSERROR_OK) {
+			nsoptions_default = NULL;
+		}
+	} else {
+		res = nsoption_free(defs);
 	}
 
-	nsoption_free(defs);
-
-	return NSERROR_OK;
+	return res;
 }
 
+
 /* exported interface documented in utils/nsoption.h */
 nserror
 nsoption_read(const char *path, struct nsoption_s *opts)
 {
-	char s[100];
+	char s[NSOPTION_MAX_LINE_LEN];
 	FILE *fp;
 	struct nsoption_s *defs;
 
@@ -596,42 +729,25 @@ nsoption_read(const char *path, struct nsoption_s *opts)
 		opts = nsoptions;
 	}
 
-	/** @todo is this and API bug not being a parameter */
+	/**
+	 * @todo is this an API bug not being a parameter
+	 */
 	defs = nsoptions_default;
 
+	if ((opts == NULL) || (defs == NULL)) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
 	fp = fopen(path, "r");
 	if (!fp) {
-		LOG(("Failed to open file '%s'", path));
+		NSLOG(netsurf, INFO, "Failed to open file '%s'", path);
 		return NSERROR_NOT_FOUND;
 	}
 
-	LOG(("Successfully opened '%s' for Options file", path));
+	NSLOG(netsurf, INFO, "Successfully opened '%s' for Options file", path);
 
-	while (fgets(s, 100, fp)) {
-		char *colon, *value;
-		unsigned int idx;
-
-		if ((s[0] == 0) || (s[0] == '#')) {
-			continue;
-		}
-
-		colon = strchr(s, ':');
-		if (colon == 0) {
-			continue;
-		}
-
-		s[strlen(s) - 1] = 0;  /* remove \n at end */
-		*colon = 0;  /* terminate key */
-		value = colon + 1;
-
-		for (idx = 0; opts[idx].key != NULL; idx++) {
-			if (strcasecmp(s, opts[idx].key) != 0) {
-				continue;
-			}
-
-			strtooption(value, &opts[idx]);
-			break;
-		}
+	while (fgets(s, NSOPTION_MAX_LINE_LEN, fp)) {
+		optionline(opts, s, strlen(s));
 	}
 
 	fclose(fp);
@@ -665,9 +781,14 @@ nsoption_write(const char *path,
 		defs = nsoptions_default;
 	}
 
+	if ((opts == NULL) || (defs == NULL)) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
 	fp = fopen(path, "w");
 	if (!fp) {
-		LOG(("failed to open file '%s' for writing", path));
+		NSLOG(netsurf, INFO, "failed to open file '%s' for writing",
+		      path);
 		return NSERROR_NOT_FOUND;
 	}
 
@@ -686,10 +807,13 @@ nsoption_dump(FILE *outf, struct nsoption_s *opts)
 		return NSERROR_BAD_PARAMETER;
 	}
 
-	/* check to see if global table selected */
+	/* check to see if global table selected and available */
 	if (opts == NULL) {
 		opts = nsoptions;
 	}
+	if (opts == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
 
 	return nsoption_output(outf, opts, NULL, true);
 }
@@ -706,10 +830,17 @@ nsoption_commandline(int *pargc, char **argv, struct nsoption_s *opts)
 	int mv_loop;
 	unsigned int entry_loop;
 
-	/* check to see if global table selected */
+	if ((pargc == NULL) || (argv == NULL)) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	/* check to see if global table selected and available */
 	if (opts == NULL) {
 		opts = nsoptions;
 	}
+	if (opts == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
 
 	while (idx < *pargc) {
 		arg = argv[idx];
@@ -737,7 +868,7 @@ nsoption_commandline(int *pargc, char **argv, struct nsoption_s *opts)
 
 		/* arg+arglen is the option to set, val is the value */
 
-		LOG(("%.*s = %s", arglen, arg, val));
+		NSLOG(netsurf, INFO, "%.*s = %s", arglen, arg, val);
 
 		for (entry_loop = 0;
 		     entry_loop < NSOPTION_LISTEND;
@@ -757,6 +888,8 @@ nsoption_commandline(int *pargc, char **argv, struct nsoption_s *opts)
 	}
 	*pargc -= (idx - 1);
 
+	nsoption_validate(opts, nsoptions_default);
+
 	return NSERROR_OK;
 }
 
@@ -771,14 +904,22 @@ nsoption_snoptionf(char *string,
 	int fmtc = 0; /* current index into format string */
 	struct nsoption_s *option;
 
+	if (fmt == NULL) {
+		return -1;
+	}
+
 	if (option_idx >= NSOPTION_LISTEND) {
 		return -1;
 	}
 
-	option = &nsoptions[option_idx]; /* assume the global table */
-	if (option == NULL || option->key == NULL)
+	if (nsoptions == NULL) {
 		return -1;
+	}
 
+	option = &nsoptions[option_idx]; /* assume the global table */
+	if (option == NULL || option->key == NULL) {
+		return -1;
+	}
 
 	while ((slen < size) && (fmt[fmtc] != 0)) {
 		if (fmt[fmtc] == '%') {
